import { useHistory, useLocation } from 'react-router-dom';
import { useState, useEffect } from 'react';

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

var initialValuesWrappers = [];
var addInitialValues = function addInitialValues(initialValues, index) {
  var wrapper = {
    initialValues: initialValues,
    index: index
  };
  initialValuesWrappers = [].concat(initialValuesWrappers.filter(function (v) {
    return v.index !== index;
  }), [wrapper]);
};
var removeInitialValues = function removeInitialValues(index) {
  initialValuesWrappers = initialValuesWrappers.filter(function (v) {
    return v.index !== index;
  });
};
var getInitialValues = function getInitialValues() {
  var initialValues = {};
  initialValuesWrappers.sort(function (a, b) {
    return a.index - b.index;
  }).forEach(function (initialValuesWrapper) {
    initialValues = _extends({}, initialValues, initialValuesWrapper.initialValues);
  });
  return initialValues;
};

function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

// Copyright Joyent, Inc. and other Node contributors.

// If obj.hasOwnProperty has been overridden, then calling
// obj.hasOwnProperty(prop) will break.
// See: https://github.com/joyent/node/issues/1707
function hasOwnProperty(obj, prop) {
  return Object.prototype.hasOwnProperty.call(obj, prop);
}

var decode = function(qs, sep, eq, options) {
  sep = sep || '&';
  eq = eq || '=';
  var obj = {};

  if (typeof qs !== 'string' || qs.length === 0) {
    return obj;
  }

  var regexp = /\+/g;
  qs = qs.split(sep);

  var maxKeys = 1000;
  if (options && typeof options.maxKeys === 'number') {
    maxKeys = options.maxKeys;
  }

  var len = qs.length;
  // maxKeys <= 0 means that we should not limit keys count
  if (maxKeys > 0 && len > maxKeys) {
    len = maxKeys;
  }

  for (var i = 0; i < len; ++i) {
    var x = qs[i].replace(regexp, '%20'),
        idx = x.indexOf(eq),
        kstr, vstr, k, v;

    if (idx >= 0) {
      kstr = x.substr(0, idx);
      vstr = x.substr(idx + 1);
    } else {
      kstr = x;
      vstr = '';
    }

    k = decodeURIComponent(kstr);
    v = decodeURIComponent(vstr);

    if (!hasOwnProperty(obj, k)) {
      obj[k] = v;
    } else if (Array.isArray(obj[k])) {
      obj[k].push(v);
    } else {
      obj[k] = [obj[k], v];
    }
  }

  return obj;
};

// Copyright Joyent, Inc. and other Node contributors.

var stringifyPrimitive = function(v) {
  switch (typeof v) {
    case 'string':
      return v;

    case 'boolean':
      return v ? 'true' : 'false';

    case 'number':
      return isFinite(v) ? v : '';

    default:
      return '';
  }
};

var encode = function(obj, sep, eq, name) {
  sep = sep || '&';
  eq = eq || '=';
  if (obj === null) {
    obj = undefined;
  }

  if (typeof obj === 'object') {
    return Object.keys(obj).map(function(k) {
      var ks = encodeURIComponent(stringifyPrimitive(k)) + eq;
      if (Array.isArray(obj[k])) {
        return obj[k].map(function(v) {
          return ks + encodeURIComponent(stringifyPrimitive(v));
        }).join(sep);
      } else {
        return ks + encodeURIComponent(stringifyPrimitive(obj[k]));
      }
    }).join(sep);

  }

  if (!name) return '';
  return encodeURIComponent(stringifyPrimitive(name)) + eq +
         encodeURIComponent(stringifyPrimitive(obj));
};

var Q__projects_reactLocationQuery_node_modules_querystring = createCommonjsModule(function (module, exports) {

exports.decode = exports.parse = decode;
exports.encode = exports.stringify = encode;
});

var history;
var setHistory = function setHistory(h) {
  return history = h;
};
var getHistory = function getHistory() {
  return history;
};
var getLocation = function getLocation() {
  return history.location;
};

var normalizeForUser = function normalizeForUser(values, initialValues) {
  if (initialValues === void 0) {
    initialValues = getInitialValues();
  }

  var normalized = {};
  Object.keys(values).forEach(function (key) {
    var value = values[key];
    var initialValue = initialValues[key];

    switch (initialValue.type) {
      case 'boolean':
        normalized[key] = normalizeBoolean(value);
        break;

      case 'number':
        normalized[key] = normalizeNumber(value);
        break;

      case 'array':
        normalized[key] = normalizeArray(value, initialValue);
        break;

      case 'json':
        normalized[key] = normalizeJson(value);
        break;

      case 'string':
        normalized[key] = normalizeString(value);
        break;

      default:
        normalized[key] = value;
        break;
    }
  });
  return normalized;
};

var normalizeBoolean = function normalizeBoolean(value) {
  if (typeof value === 'object' && 'type' in value) return value.initial;
  return value === 'true';
};

var normalizeNumber = function normalizeNumber(value) {
  if (typeof value === 'object' && 'type' in value) return value.initial;
  return parseInt(value.toString());
};

var normalizeArray = function normalizeArray(value, initialValue) {
  var normalizeArray = function normalizeArray(type, array) {
    if (typeof value === 'object' && 'type' in value) return value.initial;
    var func;

    switch (type) {
      case 'boolean':
        func = function func(a) {
          return a === true || a === 'true';
        };

        break;

      case 'number':
        func = function func(a) {
          return parseInt(a.toString());
        };

        break;
    }

    if (func) return array.map(func);
    return array;
  };

  var array = Array.isArray(value) ? value : [value];
  return normalizeArray(initialValue.arrayType, array);
};

var normalizeJson = function normalizeJson(value) {
  try {
    return JSON.parse(value);
  } catch (e) {
    return {};
  }
};

var normalizeString = function normalizeString(value) {
  if (typeof value === 'object' && 'type' in value) return value.initial;
  return value;
};

var normalizeForLocation = function normalizeForLocation(queryValues) {
  var initialValues = getInitialValues();
  var locationValues = {};
  Object.keys(initialValues).forEach(function (key) {
    var value = queryValues[key];
    var initialValue = initialValues[key];

    switch (initialValue.type) {
      case 'json':
        locationValues[key] = normalizeJson$1(value);
        break;

      case 'string':
      default:
        locationValues[key] = normalizeString$1(value);
        break;
    }

    if (initialValue.hideIfInitial) {
      if (typeof value === 'object' && !Array.isArray(value) || compareValues(value, initialValue)) {
        delete locationValues[key];
      }
    }
  });
  return locationValues;
};

var compareValues = function compareValues(value, initialValue) {
  switch (initialValue.type) {
    case 'boolean':
      return initialValue.initial === (value === 'true');

    case 'number':
      return initialValue.initial === parseInt(value);

    case 'json':
      return JSON.stringify(initialValue.initial) === value;

    case 'string':
    default:
      return initialValue.initial === value;
  }
};

var normalizeString$1 = function normalizeString(value) {
  if (typeof value === 'object' && 'type' in value) return value.initial;
  return value;
};

var normalizeJson$1 = function normalizeJson(value) {
  if (typeof value === 'object' && 'type' in value) return JSON.stringify(value.initial);
  if (typeof value === 'object') return JSON.stringify(value);
  return value;
};

var extractQueryByInitialValues = function extractQueryByInitialValues(query, defaultValues) {
  var result = {};
  Object.keys(defaultValues).forEach(function (key) {
    result[key] = query[key] ? query[key] : defaultValues[key];
  });
  return result;
};
var parseQuery = function parseQuery(query) {
  return Q__projects_reactLocationQuery_node_modules_querystring.parse(query === '' ? '' : query.substring(1));
};
var stringifyQuery = function stringifyQuery(query) {
  return Q__projects_reactLocationQuery_node_modules_querystring.stringify(query);
};
var readQuery = function readQuery() {
  var location = getLocation();
  return parseQuery(location.search);
};
var writeQuery = function writeQuery(query) {
  var history = getHistory();
  var location = getLocation();

  if (Object.keys(query).length === 0) {
    history.replace(location.pathname);
  } else {
    history.replace(location.pathname + "?" + stringifyQuery(normalizeForLocation(query)));
  }
};

var calculateLocationPath = function calculateLocationPath() {
  var queryValues = readQuery();
  writeQuery(_extends({}, getInitialValues(), queryValues));
};
var setQueryField = function setQueryField(field, value) {
  var queryValues = _extends({}, readQuery());

  queryValues[field] = value;
  writeQuery(_extends({}, getInitialValues(), queryValues));
};

var currentIndex = 0;
var useIndex = function useIndex() {
  var _useState = useState(currentIndex++),
      index = _useState[0];

  useEffect(function () {
    return function () {
      currentIndex--;
    };
  }, []);
  return index;
};

var useLocationQueryExtend = function useLocationQueryExtend(initialValues) {
  var index = useIndex();
  setHistory(useHistory());
  useLocation();
  addInitialValues(initialValues, index);
  useEffect(function () {
    addInitialValues(initialValues, index);
    calculateLocationPath();
    return function () {
      removeInitialValues(index);
      calculateLocationPath();
    };
  }, [JSON.stringify(initialValues)]);
  var currentInitialNormalizedValues = initialValues;
  var initialNormalizedValues = getInitialValues();
  var locationQuery = readQuery();
  var query = extractQueryByInitialValues(locationQuery, currentInitialNormalizedValues);
  return {
    fullQuery: normalizeForUser(extractQueryByInitialValues(locationQuery, initialNormalizedValues)),
    query: normalizeForUser(query, initialValues),
    setQueryField: function setQueryField$1(field, value) {
      return setQueryField(field, value);
    }
  };
};

var useLocationQuery = function useLocationQuery(initialValues) {
  if (initialValues === void 0) {
    initialValues = {};
  }

  return useLocationQueryExtend(initialValues);
};

var useLocationField = function useLocationField(name, value) {
  var _ref;

  var _useLocationQuery = useLocationQuery(value ? (_ref = {}, _ref[name] = value, _ref) : {}),
      fullQuery = _useLocationQuery.fullQuery,
      setQueryField = _useLocationQuery.setQueryField;

  return [fullQuery[name], function (value) {
    return setQueryField(name, value);
  }];
};

export { useLocationField, useLocationQuery, useLocationQueryExtend };
//# sourceMappingURL=index.modern.js.map
